require 'ice_cube'

# TODO Take advantage of the ice_cube gem
#      ::IceCube::Rule.weekly.day(:friday).day_of_month(13).month_of_year(:october).to_yaml ?

module ResourcePlanner
  class Event < ActiveRecord::Base
    has_event_calendar

    belongs_to :parent_event, :class_name => '::ResourcePlanner::Event', :dependent => :destroy

    has_many :recurring_event_exceptions, :class_name => '::ResourcePlanner::Event', :foreign_key => 'parent_event_id'
    has_many :event_subscriptions

    #----------------------------------------------------------------------------

    #attr_accessible :parent_event, :parent_event_id
    #attr_accessible :name, :start_at, :end_at, :all_day
    #attr_accessible :ical_expression

    #----------------------------------------------------------------------------

    # data required for creating iCal definition
    attr_accessor :dtstart
    attr_accessor :dtend
    attr_accessor :duration
    attr_accessor :exdate
    attr_accessor :rdate
    attr_accessor :rrule
    attr_accessor :timezone

    # Recurrence rules
    attr_accessor :until
    attr_accessor :count
    attr_accessor :interval
    attr_accessor :bysecond
    attr_accessor :byminute
    attr_accessor :byhour
    attr_accessor :byday
    attr_accessor :bymonthday
    attr_accessor :byyearday
    attr_accessor :byweekno
    attr_accessor :bymonth
    attr_accessor :bysetpos
    attr_accessor :wkst
    attr_accessor :freq

    #---------- Callbacks

    before_save     :before_save_handler

    #----------------------------------------------------------------------------

    # TBD: Remove this when EventCalendar is fixed
    # Get the events overlapping the given start and end dates
    def self.events_for_date_range(start_d, end_d, find_options = {})
      self.where(find_options)
      .where("(? <= #{self.quoted_table_name}.#{self.end_at_field}) AND (#{self.quoted_table_name}.#{self.start_at_field}< ?)", start_d.to_time.utc, end_d.to_time.utc)
      .order("#{self.quoted_table_name}.#{self.start_at_field} ASC")
    end

    #----------------------------------------------------------------------------

    def to_s
      "#{name} - #{start_at}"
    end

    def full_details
      ::IceCube::Rule::from_yaml(ical_expression)
    end

    def all_day?
      false
    end

    #----------------------------------------------------------------------------

    private

    def to_ical_expression(count = 1)
      case repeating_name
      when 'secondly'
        ::IceCube::Rule.secondly(count).to_yaml
      when 'minutely'
        ::IceCube::Rule.minutely(count).to_yaml
      when 'hourly'
        ::IceCube::Rule.hourly(count).to_yaml
      when 'daily'
        ::IceCube::Rule.daily(count).to_yaml
      when 'weekly'
        ::IceCube::Rule.weekly(count).to_yaml
      when 'monthly'
        ::IceCube::Rule.monthly(count).to_yaml
      when 'annually'
        ::IceCube::Rule.yearly(count).to_yaml
      else
        ''
      end
    end

    #---------- Callback handlers

    def before_save_handler
      #ical_expression = ::IceCube::Rule.weekly.day(:friday).day_of_month(13).month_of_year(:october).to_yaml
    end
  end
end
